require 'rubygems'
require 'redis'
require 'securerandom'
require 'forwardable'
require 'singularity/configuration'

module Singularity
  @config = Configuration.new

  class << self
    extend Forwardable

    def configure
      if block_given?
        yield @config
      else
        @config
      end
    end

    def_delegators :@config, :auth_path, :session_prefix, :profile_prefix, :token_prefix, :warden_user_key
    def_delegators :@config, :auth_path=, :session_prefix=, :profile_prefix=, :token_prefix=, :warden_user_key=

    def generate_session_token(session_id, session, options = {})
      if session.key?("_token_expires") && session["_token_expires"].to_i > Time.now.to_i
        unless session.nil? || session["_session_id"] != session_id
          return session["_token"]
        else
          redis.del token_key(session['_token'])
        end
      end

      session_token    = SecureRandom.hex(32)
      token_expiration = options.fetch(:expires){ redis.ttl session_key(session_id) }
      token_expiration = 1_800 if token_expiration < 1

      redis.setex token_key(session_token), token_expiration, session_id

      session["_session_id"]    = session_id
      session["_token"]         = session_token
      session["_token_expires"] = Time.now.to_i + token_expiration

      session_token
    end

    def session_key(id)
      Singularity.session_prefix ? "#{ Singularity.session_prefix }:#{ id }" : id
    end

    def token_key(token)
      "#{ Singularity.token_prefix }:#{ token }"
    end

    def find_session_id_by_token(session_token)
      session_id = redis.get token_key(session_token)
      return nil if session_id.nil?
      redis.exists(session_key(session_id)) ? session_id : nil
    end

    def fetch_key_from_session(session_id, key)
      redis.hget session_key(session_id), key if session_id
    end

    def store_key_in_session(session_id, key, value)
      redis.hset session_key(session_id), key, value if session_id
    end

    def destroy_session(*ids)
      redis.del( *ids.map{ |id| session_key(id) } )
    end

    def destroy_token(*tokens)
      redis.del( *tokens.map{ |t| token_key(t) } )
    end

    def redis
      @redis ||= Redis.current
    end
  end
end
