require 'openssl'
require 'rack/utils'

module Singularity
  class PasswordHash
    attr_reader :digest

    def initialize(digest = nil)
      @digest = digest
    end

    def self.iterations; 20_000; end

    def self.digest_algorithm
      OpenSSL::Digest::SHA512.new
    end

    def self.key_length
      digest_algorithm.length
    end

    def self.generate(password, salt, options = {})
      iters = options.fetch(:iterations, iterations)
      OpenSSL::PKCS5.pbkdf2_hmac(password, salt, iters, key_length, digest_algorithm).unpack('H*').first
    end

    def matches?(password, salt, options = {})
      Rack::Utils.secure_compare digest, self.class.generate(password, salt, options)
    end
  end
end
